package Algorithms.Animated.BK;

import Algorithms.Animated.AlgorithmStep;
import Algorithms.Animated.AnnimatedAlgorithm;
import Algorithms.Animated.AnnimationController;
import Model.LayeredGraphNode;

public class BKNodePlacement extends AnnimatedAlgorithm {

	/*
	 * Private data structures to store the process of the algorithm
	 */
	
	private enum State
	{
		LAYOUT1,
		LAYOUT2,
		LAYOUT3,
		LAYOUT4,
		COMBINE
	}
	
	private State state;
	private Layout layouts[];
	private Combine combine;
	
	public BKNodePlacement(AnnimationController controller, LayeredGraphNode graph) {
		super(controller, graph);
		state = State.LAYOUT1;
		layouts = new Layout[ 4 ];
		layouts[ 0 ] = new Layout( Layout.LayoutType.TOP_BOTTOM_LEFT, graph );
		layouts[ 1 ] = new Layout( Layout.LayoutType.TOP_BOTTOM_RIGHT, graph );
		layouts[ 2 ] = new Layout( Layout.LayoutType.BOTTOM_TOP_LEFT, graph );
		layouts[ 3 ] = new Layout( Layout.LayoutType.BOTTOM_TOP_RIGHT, graph );
		combine = new Combine( graph );
	}

	@Override
	public StepStatus forwardStep() {
		switch( state )
		{
		case LAYOUT1:
			if( layouts[ 0 ].forwardStep() == AlgorithmStep.StepStatus.FINISHED )
				state = State.LAYOUT2;
			break;
		case LAYOUT2:
			if( layouts[ 1 ].forwardStep() == AlgorithmStep.StepStatus.FINISHED )
				state = State.LAYOUT3;
			break;
		case LAYOUT3:
			if( layouts[ 2 ].forwardStep() == AlgorithmStep.StepStatus.FINISHED )
				state = State.LAYOUT4;
			break;
		case LAYOUT4:
			if( layouts[ 3 ].forwardStep() == AlgorithmStep.StepStatus.FINISHED )
				state = State.COMBINE;
			break;
		case COMBINE:
			return combine.forwardStep();
		}
		return StepStatus.UNFINISHED;
	}

	@Override
	public StepStatus backwardStep() {
		switch( state )
		{
		case LAYOUT1:
			return layouts[ 0 ].backwardStep();
		case LAYOUT2:
			if( layouts[ 1 ].backwardStep() == AlgorithmStep.StepStatus.FINISHED )
				state = State.LAYOUT1;
			break;
		case LAYOUT3:
			if( layouts[ 2 ].backwardStep() == AlgorithmStep.StepStatus.FINISHED )
				state = State.LAYOUT2;
			break;
		case LAYOUT4:
			if( layouts[ 3 ].backwardStep() == AlgorithmStep.StepStatus.FINISHED )
				state = State.LAYOUT3;
			break;
		case COMBINE:
			if( combine.backwardStep() == AlgorithmStep.StepStatus.FINISHED )
				state = State.LAYOUT4;
			break;
		}
		return StepStatus.UNFINISHED;
	}
}
