package Algorithms.Animated.BK;

import java.awt.Color;
import java.util.ArrayList;

import Algorithms.Animated.AlgorithmStep;
import Algorithms.Animated.BackwardAction;
import Model.LayeredGraphEdge;
import Model.LayeredGraphNode;

public class BlockCalc implements AlgorithmStep {

	private int layerIndex;
	private int nodeIndex;
	private int r;
	private LayeredGraphNode graph;
	private ArrayList< ArrayList< BKNodePlacement > > subgraphAlgs;
	private ArrayList< BackwardAction > backwards;
	int step;
	
	public BlockCalc( LayeredGraphNode graph )
	{
		step = 0;
		this.graph = graph;
		layerIndex = 0;
		nodeIndex = 0;
		r = 0;
		subgraphAlgs = new ArrayList<>();
		for( ArrayList<LayeredGraphNode> l : graph.getContainedLayers() )
		{
			ArrayList< BKNodePlacement > algs = new ArrayList<>();
			for( int i = 0; i < l.size(); i++ )
				algs.add( null );
			subgraphAlgs.add( algs );
		}
		backwards = new ArrayList<>();
	}
	
	@Override
	public StepStatus forwardStep() {
		System.out.println( "test" );
		LayeredGraphNode current = graph.getContainedLayers().get( layerIndex ).get( nodeIndex );
		current.setSelected();
		if( current.getContainedNodes().size() > 0 )
		{
			if( subgraphAlgs.get( layerIndex ).get( nodeIndex ) == null )
				subgraphAlgs.get( layerIndex ).set( nodeIndex, new BKNodePlacement( null, current ) );
			if( subgraphAlgs.get( layerIndex ).get( nodeIndex ).forwardStep() == StepStatus.UNFINISHED )
				return StepStatus.UNFINISHED;
		}
		ArrayList< LayeredGraphEdge > incommingEdges = current.getIncomingEdges();
		if( incommingEdges.size() == 0 )
		{
			//current.update();
			backwards.add( 0, () -> {
				System.out.println( "Performing Empty Backwards Step..." );
			});
			return calcNextState();
		}
		int[] ms = {(incommingEdges.size() + 1) / 2, (int)( (incommingEdges.size() + 1) / 2.0 + 0.5 )};
		boolean backwardsAdded = false;
		for( int m : ms )
		{
		    if( current.getAlignedTo() == current )
		    {
                LayeredGraphNode u = incommingEdges.get( m - 1 ).getSources().get( 0 );
		        // TODO detect conflicts
		        if( r < graph.getContainedLayers().get( layerIndex - 1 ).indexOf( u ) )
		        {
		            LayeredGraphNode oldAlignU = u.getAlignedTo();
		            Color oldColorCurrent = current.getColor();
		            LayeredGraphNode oldRootCurrent = current.getRoot();
		            LayeredGraphNode oldAlignCurrent = current.getAlignedTo();
		            int oldR = r;
		            u.setAlignTo( current );
		            current.setColor( u.getRoot().getColor() );
		            current.setRoot( u.getRoot() );
		            current.setAlignTo( current.getRoot() );
		            r = graph.getContainedLayers().get( layerIndex - 1 ).indexOf( u );
		            int oldStep = step++;
		            backwards.add( 0, () -> {
		                System.out.println( "Stepping Backwards... (Step " + oldStep + ")" );
		                u.setAlignTo( oldAlignU );
		                current.setColor( oldColorCurrent );
		                current.setRoot( oldRootCurrent );
		                current.setAlignTo( oldAlignCurrent );
		                r = oldR;
		            });
		            backwardsAdded = true;
		        }
		    }
		}
		if( !backwardsAdded )
		{
    		backwards.add( 0, () -> {
                System.out.println( "Performing Empty Backwards Step..." );
    		});
		}
		//current.update();
		return calcNextState();
	}
	
	private StepStatus calcNextState()
	{
		if( layerIndex >= graph.getContainedLayers().size() - 1 )
		{
			if( nodeIndex >= graph.getContainedLayers().get( layerIndex ).size() -1 )
				return StepStatus.FINISHED;
		}
		nodeIndex++;
		if( nodeIndex >= graph.getContainedLayers().get( layerIndex ).size() )
		{
			layerIndex++;
			nodeIndex = 0;
			int oldR = r;
			r = 0;
			backwards.add(0, ()->{
			   this.r = oldR;
			});
		}
		return StepStatus.UNFINISHED;
	}

	@Override
	public StepStatus backwardStep() {
		if( subgraphAlgs.get( layerIndex ).get( nodeIndex ) != null )
		{
			if( subgraphAlgs.get( layerIndex ).get( nodeIndex ).backwardStep() == StepStatus.UNFINISHED )
			{
				LayeredGraphNode current = graph.getContainedLayers().get( layerIndex ).get( nodeIndex );
				current.setSelected();
				//current.update();
				return StepStatus.UNFINISHED;
			}
		}
		StepStatus status = calcBeforeState();
		if( status == StepStatus.FINISHED )
			return status;
		LayeredGraphNode current = graph.getContainedLayers().get( layerIndex ).get( nodeIndex );
		current.setSelected();
		//current.update();
		if( !backwards.isEmpty() )
		{
			backwards.get( 0 ).reverse();
			backwards.remove( 0 );
		}
		return status;
	}
	
	private StepStatus calcBeforeState()
	{
		if( layerIndex == 0 )
		{
			if( nodeIndex == 0 )
				return StepStatus.FINISHED;
		}
		nodeIndex--;
		if( nodeIndex < 0 )
		{
			layerIndex--;
	        backwards.get( 0 ).reverse();
	        backwards.remove( 0 );
			nodeIndex = graph.getContainedLayers().get( layerIndex ).size() - 1;
		}
		return StepStatus.UNFINISHED;
	}
}
