package View;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.*;

import Model.LayeredGraphNode;

public class NodeView extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private LayeredGraphNode model;
	
	public NodeView( LayeredGraphNode model ) {
		this.model = model;
		model.setView( this );
	}
	
	@Override
	public int getX()
	{
	    return (int)model.getX();
	}
    
    @Override
    public int getY()
    {
        return (int)model.getY();
    }
    
    @Override
    public int getWidth()
    {
        return (int)model.getWidth();
    }
    
    @Override
    public int getHeight()
    {
        return (int)model.getHeight();
    }
    
    @Override
    public void paint( Graphics g )
    {
        double scale = Math.min( super.getWidth() / (double)getWidth(), super.getHeight() / (double)getHeight());
        ((Graphics2D)g).scale( scale, scale );
        paintComponent( g );
        for( Component c : getComponents() )
            c.paint( g.create( 
                    c.getX() + 25, 
                    c.getY() + 25, 
                    Math.min( getWidth() - 25, c.getWidth() + c.getX() + 25 ), 
                    Math.min( getHeight() - 25, c.getHeight() + c.getY() + 25 ) ) );
    }

	@Override
	public void paintComponent( Graphics g )
	{
		if( model.isSelected() )
		{
			g.setColor( Color.GRAY );
			g.fillRect( 0, 0, getWidth()-1, getHeight()-1 );
		}
		else
		{
			g.setColor( Color.WHITE );
			g.fillRect( 0, 0, getWidth()-1, getHeight()-1 );
		}
		Graphics2D g2 = (Graphics2D)g;
		g2.setColor( model.getColor() );
		g2.setStroke(new BasicStroke(5));
		g2.drawRect( 0, 0, getWidth()-1, getHeight()-1 );
	}
}
